package org.xbib.interlibrary.catalog.elasticsearch.bib;

import org.xbib.content.rdf.RdfContentBuilder;
import org.xbib.content.rdf.RdfXContentFactory;
import org.xbib.content.rdf.RouteRdfXContentParams;
import org.xbib.interlibrary.catalog.builder.marc.DefaultWorkerPoolMarcRecordListener;
import org.xbib.interlibrary.catalog.elasticsearch.ElasticsearchIndexDefinition;
import org.xbib.interlibrary.catalog.elasticsearch.oai.ElasticsearchToolOAI;
import org.xbib.interlibrary.catalog.entities.CatalogEntityBuilder;
import org.xbib.interlibrary.catalog.entities.CatalogEntityBuilderState;
import org.xbib.interlibrary.catalog.entities.CatalogEntityWorkerPool;
import org.xbib.marc.Marc;
import org.xbib.marc.dialects.pica.PicaXMLContentHandler;
import org.xbib.marc.transformer.value.MarcValueTransformers;
import org.xbib.settings.Settings;

import java.io.IOException;
import java.io.InputStream;
import java.text.Normalizer;
import java.util.concurrent.atomic.AtomicLong;

public class AddressesOAI extends ElasticsearchToolOAI {

    public AddressesOAI() {
    }

    @Override
    public AddressesOAIWorkerPool newPool(Settings settings) throws IOException {
        return new AddressesOAIWorkerPool(settings);
    }

    public class AddressesOAIWorkerPool extends ElasticsearchOAIWorkerPool {

        public AddressesOAIWorkerPool(Settings settings) throws IOException {
            super(settings);
        }

        @Override
        public AddressesOAIWorker newWorker(String name, int count) throws IOException {
            return new AddressesOAIWorker(settings, this);
        }
    }

    public class AddressesOAIWorker extends ElasticsearchOAIWorker {

        private final CatalogEntityWorkerPool<CatalogEntityBuilder> catalogEntityWorkerPool;

        public AddressesOAIWorker(Settings settings, AddressesOAIWorkerPool pool) throws IOException {
            super(settings, pool);
            this.catalogEntityWorkerPool = new CatalogEntityWorkerPool<>(settings,
                    new DefaultWorkerPoolMarcRecordListener(), AddressesOAICatalogEntityBuilder::new, pool);
        }

        @Override
        protected void process(InputStream inputStream) {
            try {
                PicaXMLContentHandler contentHandler = new PicaXMLContentHandler();
                contentHandler.setFormat("Pica");
                contentHandler.setType("XML");
                contentHandler.setMarcListener(catalogEntityWorkerPool);
                MarcValueTransformers marcValueTransformers = new MarcValueTransformers();
                marcValueTransformers.setMarcValueTransformer(value ->
                        Normalizer.normalize(value, Normalizer.Form.NFKC));
                Marc.builder()
                        .disableControlFields()
                        .setInputStream(inputStream)
                        .setContentHandler(contentHandler)
                        .setMarcValueTransformers(marcValueTransformers)
                        .build()
                        .xmlReader()
                        .parse();
            } catch (IOException e) {
                throw new IllegalStateException(e);
            }
        }

        private class AddressesOAICatalogEntityBuilder extends CatalogEntityBuilder {

            AddressesOAICatalogEntityBuilder(Settings settings, AtomicLong counter) throws IOException {
                super(settings, counter);
            }

            @Override
            public void afterFinishState(CatalogEntityBuilderState state) throws IOException {
                ElasticsearchIndexDefinition elasticsearchIndexDefinition = pool.getOutputIndexDefinitionMap().get("bib");
                if (elasticsearchIndexDefinition == null) {
                    throw new IOException("no 'bib' index definition configured");
                }
                RouteRdfXContentParams routeRdfXContentParams =
                        new RouteRdfXContentParams(catalogEntityWorkerPool.getNamespaceContext(),
                                elasticsearchIndexDefinition.getFullIndexName(),
                                elasticsearchIndexDefinition.getType());
                routeRdfXContentParams.setHandler((content, p) ->
                        pool.bulkIndex(elasticsearchIndexDefinition, state.getRecordIdentifier(), false, content)
                );
                if (settings.get("collection") != null) {
                    state.getResource().add("collection", settings.get("collection"));
                }
                try (RdfContentBuilder<?> builder = RdfXContentFactory.routeRdfXContentBuilder(routeRdfXContentParams)) {
                    builder.receive(state.getResource());
                }
            }
        }
    }
}
