package org.xbib.interlibrary.catalog.elasticsearch.biomedcentral;

import java.io.IOException;
import java.util.Map;
import org.xbib.content.core.XContentHelper;
import org.xbib.content.rdf.RdfConstants;
import org.xbib.content.rdf.RdfContentBuilder;
import org.xbib.content.rdf.RdfXContentFactory;
import org.xbib.content.rdf.RdfXContentParams;
import org.xbib.content.rdf.Resource;
import org.xbib.content.resource.IRI;
import org.xbib.content.resource.IRINamespaceContext;
import org.xbib.interlibrary.catalog.elasticsearch.oai.ElasticsearchToolOAI;
import org.xbib.settings.Settings;

/**
 * OAI harvester for BioMed Central.
 */
public class BioMedCentralOAI extends ElasticsearchToolOAI {

    public BioMedCentralOAI() {
    }

    @Override
    public BioMedCentralOAIWorkerPool newPool(Settings settings) throws IOException {
        return new BioMedCentralOAIWorkerPool(settings);
    }

    public class BioMedCentralOAIWorkerPool extends ElasticsearchOAIWorkerPool {

        public BioMedCentralOAIWorkerPool(Settings settings) throws IOException {
            super(settings);
        }

        @Override
        public BioMedCentralOAIWorker newWorker(String name, int count) {
            return new BioMedCentralOAIWorker(settings, this);
        }
    }

    public class BioMedCentralOAIWorker extends ElasticsearchOAIWorker {

        public BioMedCentralOAIWorker(Settings settings, ElasticsearchOAIWorkerPool pool) {
            super(settings, pool);
        }

        @Override
        protected String convert(String id, String content) throws IOException {
            RdfXContentParams params = new RdfXContentParams(createNamespaceContext());
            Map<String, ?> map = XContentHelper.convertToMap(content);
            BioMedCentralMapper mapper = new BioMedCentralMapper();
            Resource resource = mapper.map(map);
            try (RdfContentBuilder<?> builder = RdfXContentFactory.rdfXContentBuilder(params)) {
                builder.receive(IRI.create(id));
                builder.receive(resource);
            }
            return params.getGenerator().get();
        }
    }

    private static IRINamespaceContext createNamespaceContext() {
        IRINamespaceContext namespaceContext = IRINamespaceContext.newInstance();
        namespaceContext.addNamespace(RdfConstants.NS_PREFIX, RdfConstants.NS_URI);
        namespaceContext.addNamespace("dc", "http://purl.org/dc/elements/1.1/");
        namespaceContext.addNamespace("dcterms", "http://purl.org/dc/terms/");
        namespaceContext.addNamespace("foaf", "http://xmlns.com/foaf/0.1/");
        namespaceContext.addNamespace("frbr", "http://purl.org/vocab/frbr/core#");
        namespaceContext.addNamespace("fabio", "http://purl.org/spar/fabio/");
        namespaceContext.addNamespace("prism", "http://prismstandard.org/namespaces/basic/3.0/");
        return namespaceContext;
    }
}
