package org.xbib.interlibrary.catalog.elasticsearch.doaj;

import java.io.IOException;
import java.util.Map;
import org.xbib.content.core.XContentHelper;
import org.xbib.content.rdf.RdfContentBuilder;
import org.xbib.content.rdf.RdfXContentFactory;
import org.xbib.content.rdf.RdfXContentParams;
import org.xbib.content.rdf.Resource;
import org.xbib.content.resource.IRI;
import org.xbib.interlibrary.catalog.elasticsearch.oai.ElasticsearchToolOAI;
import org.xbib.settings.Settings;

/**
 * Harvest DOAJ articles from OAI and index them into Elasticsearch.
 */
public class DOAJArticle extends ElasticsearchToolOAI {

    public DOAJArticle() {
    }

    @Override
    public DOAJArticleWorkerPool newPool(Settings settings) throws IOException {
        return new DOAJArticleWorkerPool(settings);
    }

    public class DOAJArticleWorkerPool extends ElasticsearchOAIWorkerPool {

        public DOAJArticleWorkerPool(Settings settings) throws IOException {
            super(settings);
        }

        @Override
        public DOAJArticleWorker newWorker(String name, int count) {
            return new DOAJArticleWorker(settings, this);
        }
    }

    public class DOAJArticleWorker extends ElasticsearchOAIWorker {

        public DOAJArticleWorker(Settings settings, DOAJArticleWorkerPool pool) {
            super(settings, pool);
        }

        @Override
        protected String convert(String id, String content) throws IOException {
            Map<String, ?> map = XContentHelper.convertToMap(content);
            DOAJArticleMapper mapper = new DOAJArticleMapper();
            Resource resource = mapper.map(map);
            RdfXContentParams params = new RdfXContentParams(pool.getNamespaceContext());
            try (RdfContentBuilder<?> builder = RdfXContentFactory.rdfXContentBuilder(params)) {
                builder.receive(IRI.create(id));
                builder.receive(resource);
            }
            return params.getGenerator().get();
        }
    }
}
