package org.xbib.interlibrary.catalog.elasticsearch.doaj;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.xbib.content.rdf.Literal;
import org.xbib.content.rdf.Resource;
import org.xbib.content.rdf.internal.DefaultAnonymousResource;
import org.xbib.content.rdf.internal.DefaultLiteral;
import org.xbib.interlibrary.catalog.entities.ArticleVocabulary;

class DOAJArticleMapper implements ArticleVocabulary {

    private static final Logger logger = Logger.getLogger(DOAJArticleMapper.class.getName());

    public Resource map(Map<String, ?> map) throws IOException {
        Resource r = new DefaultAnonymousResource();
        map(r, null, map);
        return r;
    }

    @SuppressWarnings("unchecked")
    private void map(Resource r, String p, Map<String, ?> map) throws IOException {
        for (Map.Entry<String, ?> entry : map.entrySet()) {
            String key = entry.getKey();
            Object value = entry.getValue();
            String path = p != null ? p + "." + key : key;
            if (value instanceof Map) {
                map(r, path, (Map<String, Object>) value);
            } else if (value instanceof List) {
                for (Object o : (List) value) {
                    if (o instanceof Map) {
                        map(r, path, (Map<String, Object>) o);
                    } else {
                        map(r, path, o.toString());
                    }
                }
            } else {
                if (value != null) {
                    map(r, path, value.toString());
                }
            }
        }
    }

    public void map(Resource r, String path, String valueString) throws IOException {
        String value = valueString;
        switch (path) {
            case "title":
                r.add(DC_TITLE, value);
                break;
            case "creator":
                r.add(DC_CREATOR, value);
                break;
            case "identifier":
                if (value.startsWith("10.")) {
                    r.add(PRISM_DOI, value);
                } else if (value.startsWith("https://doaj.org/")) {
                    r.add(XBIB_DOAJ_ID, value);
                } else if (value.length() == 9 && value.indexOf('-') == 4) {
                    r.add(PRISM_ISSN, value);
                } else if (value.startsWith("http://dx.doi.org/")) {
                    value = value.substring("http://dx.doi.org/".length());
                    r.add(PRISM_DOI, value);
                } else if (value.startsWith("doi:")) {
                    value = value.substring("doi:".length());
                    r.add(PRISM_DOI, value);
                } else {
                    logger.log(Level.WARNING, "unknown identifier: " + value);
                }
                break;
            case "date":
                r.add(PRISM_PUBLICATIONDATE, value);
                if (value.length() >= 4) {
                    r.add(DC_DATE, new DefaultLiteral(value.substring(0, 4)).type(Literal.GYEAR));
                }
                break;
            case "publisher":
                r.add(DC_PUBLISHER, value);
                break;
            case "type":
                r.add(DC_TYPE, value);
                break;
            case "rights":
                r.add(DC_RIGHTS, value);
                break;
            case "description":
                value = value.replaceAll("\\<[^>]*>", "");
                r.add(DCTERMS_ABSTRACT, value);
                break;
            case "relation":
                if (!value.startsWith("https://doaj.org/")) {
                    r.add(DC_SOURCE, value);
                }
                break;
            case "source":
                r.add(DCTERMS_BIBLIOGRAPHIC_CITATION, value);
                break;
            default:
                break;
        }
    }
}
