package org.xbib.interlibrary.catalog.elasticsearch.ezb;

import org.xbib.content.rdf.RdfContentBuilder;
import org.xbib.content.rdf.RdfXContentFactory;
import org.xbib.content.rdf.RouteRdfXContentParams;
import org.xbib.interlibrary.catalog.builder.marc.DefaultWorkerPoolMarcRecordListener;
import org.xbib.interlibrary.catalog.elasticsearch.ElasticsearchIndexDefinition;
import org.xbib.interlibrary.catalog.elasticsearch.oai.ElasticsearchToolOAI;
import org.xbib.interlibrary.catalog.entities.CatalogEntityBuilder;
import org.xbib.interlibrary.catalog.entities.CatalogEntityBuilderState;
import org.xbib.interlibrary.catalog.entities.CatalogEntityWorkerPool;
import org.xbib.marc.Marc;
import org.xbib.marc.xml.MarcContentHandler;
import org.xbib.settings.Settings;

import java.io.IOException;
import java.io.InputStream;
import java.util.concurrent.atomic.AtomicLong;

public class EZBOAI extends ElasticsearchToolOAI {

    public EZBOAI() {
    }

    @Override
    public EZBOAIWorkerPool newPool(Settings settings) throws IOException {
        return new EZBOAIWorkerPool(settings);
    }

    public class EZBOAIWorkerPool extends ElasticsearchOAIWorkerPool {

        public EZBOAIWorkerPool(Settings settings) throws IOException {
            super(settings);
        }

        @Override
        public EZBOAIWorker newWorker(String name, int count) throws IOException {
            return new EZBOAIWorker(settings, this);
        }
    }

    public class EZBOAIWorker extends ElasticsearchOAIWorker {

        private final CatalogEntityWorkerPool<CatalogEntityBuilder> catalogEntityWorkerPool;

        public EZBOAIWorker(Settings settings, EZBOAIWorkerPool pool) throws IOException {
            super(settings, pool);
            this.catalogEntityWorkerPool = new CatalogEntityWorkerPool<>(settings,
                    new DefaultWorkerPoolMarcRecordListener(), EZBOAICatalogEntityBuilder::new, pool);
        }

        @Override
        protected void process(InputStream inputStream) {
            try {
                MarcContentHandler contentHandler = new MarcContentHandler();
                contentHandler.setFormat("Marc21");
                contentHandler.setType("Holdings");
                contentHandler.setMarcListener(catalogEntityWorkerPool);
                Marc.builder()
                        .setInputStream(inputStream)
                        .setContentHandler(contentHandler)
                        .build()
                        .xmlReader()
                        .parse();
            } catch (IOException e) {
                throw new IllegalStateException(e);
            }
        }

        private class EZBOAICatalogEntityBuilder extends CatalogEntityBuilder {

            EZBOAICatalogEntityBuilder(Settings settings, AtomicLong counter) throws IOException {
                super(settings, counter);
            }

            @Override
            public void afterFinishState(CatalogEntityBuilderState state) throws IOException {
                ElasticsearchIndexDefinition elasticsearchIndexDefinition = pool.getOutputIndexDefinitionMap().get("ezb");
                if (elasticsearchIndexDefinition == null) {
                    throw new IOException("no 'ezb' index definition configured");
                }
                RouteRdfXContentParams routeRdfXContentParams =
                        new RouteRdfXContentParams(pool.getNamespaceContext(),
                                elasticsearchIndexDefinition.getFullIndexName(),
                                elasticsearchIndexDefinition.getType());
                routeRdfXContentParams.setHandler((content, p) ->
                        pool.bulkIndex(elasticsearchIndexDefinition, state.getRecordIdentifier(), false, content)
                );
                if (settings.get("collection") != null) {
                    state.getResource().add("collection", settings.get("collection"));
                }
                try (RdfContentBuilder<?> builder = RdfXContentFactory.routeRdfXContentBuilder(routeRdfXContentParams)) {
                    builder.receive(state.getResource());
                }
            }
        }
    }
}
