package org.xbib.interlibrary.catalog.elasticsearch.ezb;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import org.xbib.interlibrary.api.action.worker.WorkerRequest;
import org.xbib.interlibrary.catalog.builder.ConverterInput;
import org.xbib.interlibrary.catalog.builder.StringWorkerRequest;
import org.xbib.interlibrary.catalog.elasticsearch.ElasticsearchTool;
import org.xbib.interlibrary.catalog.elasticsearch.ElasticsearchIndexDefinition;
import org.xbib.settings.Settings;

public class EZBWebFile extends ElasticsearchTool<String> {

    public EZBWebFile() {
    }

    @Override
    public EZBWebFileWorkerPool newPool(Settings settings) throws IOException {
        return new EZBWebFileWorkerPool(settings);
    }

    public class EZBWebFileWorkerPool extends ElasticsearchWorkerPool<String> {

        public EZBWebFileWorkerPool(Settings settings) throws IOException {
            super("ezb-web-file", settings);
        }

        @Override
        public EZBWebFileWorker newWorker(String name, int count) {
            return new EZBWebFileWorker(settings, this);
        }

        @Override
        public WorkerRequest<String> newWorkerRequest(Object input, String origin) {
            return input != null ? new StringWorkerRequest(input.toString(), origin) : null;
        }
    }

    public class EZBWebFileWorker extends ElasticsearchWorker<String, EZBWebFileWorkerPool> {

        private final EZBWebFileWorkerPool pool;

        public EZBWebFileWorker(Settings settings, EZBWebFileWorkerPool pool) {
            super(settings);
            this.pool = pool;
        }

        @Override
        protected void doExecute(String uri) throws IOException {
            ElasticsearchIndexDefinition indexDefinition = pool.getOutputIndexDefinitionMap().get("bib");
            try (InputStream inputStream = ConverterInput.getInputStream(uri, settings);
                 BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream,
                         StandardCharsets.UTF_8))) {
                bufferedReader.lines().forEach(line ->
                    pool.bulkIndex(indexDefinition, null, false, line));
            }
        }
    }
}
