package org.xbib.interlibrary.catalog.elasticsearch.hbz;

import java.io.IOException;
import java.io.InputStream;
import org.xbib.interlibrary.catalog.builder.MarcXchangeJSONLinesReader;
import org.xbib.interlibrary.catalog.entities.CatalogEntityBuilder;
import org.xbib.interlibrary.catalog.entities.CatalogEntityWorkerPool;
import org.xbib.settings.Settings;

/**
 * Index title/holdings from MarcXchange JSON lines files into Elasticsearch.
 */
public class TitleHoldingsMarcJSONLines extends TitleHoldings {

    @Override
    public TitleHoldingsMarcJSONLinesWorkerPool newPool(Settings settings) throws IOException {
        return new TitleHoldingsMarcJSONLinesWorkerPool(settings);
    }

    public class TitleHoldingsMarcJSONLinesWorkerPool extends TitleHoldingsWorkerPool {

        protected TitleHoldingsMarcJSONLinesWorkerPool(Settings settings) throws IOException {
            super(settings);
        }

        @Override
        public TitleHoldingsMarcJSONLinesWorker newWorker(String name, int count) {
            return new TitleHoldingsMarcJSONLinesWorker(settings, this);
        }
    }

    public class TitleHoldingsMarcJSONLinesWorker extends TitleHoldingsWorker {

        protected TitleHoldingsMarcJSONLinesWorker(Settings settings, TitleHoldingsWorkerPool pool) {
            super(settings, pool);
        }

        @Override
        protected void doExecute(InputStream inputStream,
                                 CatalogEntityWorkerPool<CatalogEntityBuilder> catalogEntityWorkerPool)
                throws IOException {
            try (MarcXchangeJSONLinesReader reader = new MarcXchangeJSONLinesReader(inputStream,
                    settings.getAsInt("buffersize", 65536), catalogEntityWorkerPool)) {
                reader.setMarcFieldTransformers(marcFieldTransformers);
                reader.setMarcValueTransformers(marcValueTransformers);
                reader.parse();
            }
        }
    }
}
