package org.xbib.interlibrary.catalog.elasticsearch.hbzalma;

import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.index.query.MatchAllQueryBuilder;
import org.xbib.interlibrary.api.action.worker.WorkerRequest;
import org.xbib.interlibrary.catalog.builder.ConverterOutput;
import org.xbib.interlibrary.catalog.builder.StringWorkerRequest;
import org.xbib.interlibrary.catalog.elasticsearch.ElasticsearchTool;
import org.xbib.interlibrary.catalog.elasticsearch.ElasticsearchIndexDefinition;
import org.xbib.marc.MarcRecord;
import org.xbib.marc.xml.MarcXchangeWriter;
import org.xbib.settings.Settings;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.zip.GZIPOutputStream;

public class HbzAlmaExport extends ElasticsearchTool<String> {

    public HbzAlmaExport() {
    }

    @Override
    public ElasticsearchWorkerPool<String> newPool(Settings settings) throws IOException {
        return new HbzAlmaExportWorkerPool(settings);
    }

    public class HbzAlmaExportWorkerPool extends ElasticsearchWorkerPool<String> {

        public HbzAlmaExportWorkerPool(Settings settings) throws IOException {
            super("hbz-alma-export", settings);
        }

        @Override
        public HbzAlmaExportWorker newWorker(String name, int count) throws IOException {
            return new HbzAlmaExportWorker(settings, this);
        }

        @Override
        public WorkerRequest<String> newWorkerRequest(Object input, String origin) {
            return input != null ? new StringWorkerRequest(input.toString(), origin) : null;
        }
    }

    public class HbzAlmaExportWorker extends ElasticsearchWorker<String, HbzAlmaExportWorkerPool> {

        private final HbzAlmaExportWorkerPool pool;

        public HbzAlmaExportWorker(Settings settings, HbzAlmaExportWorkerPool pool) {
            super(settings);
            this.pool = pool;
        }

        @Override
        protected void doExecute(String uri) throws IOException {
            ElasticsearchIndexDefinition elasticsearchIndexDefinition = pool.getInputIndexDefinitionMap().get("bib");
            if (elasticsearchIndexDefinition == null) {
                throw new IOException("no 'bib' index definition configured");
            }
            ConverterOutput.Entry entry = pool.getConverterOutput().getMap().get("path");
            if (entry == null) {
                throw new IOException("no 'path' output definition configured");
            }
            Path path = entry.getPath();
            try (MarcXchangeWriter marcXchangeWriter = new MarcXchangeWriter(new GZIPOutputStream(Files.newOutputStream(path)))) {
                marcXchangeWriter.startDocument();
                marcXchangeWriter.beginCollection();
                pool.searchClient().search(qb -> qb
                                .setIndices(elasticsearchIndexDefinition.getFullIndexName())
                                .setQuery(new MatchAllQueryBuilder()),
                        TimeValue.timeValueMillis(1000), 1000)
                        .forEach(doc -> marcXchangeWriter.record(MarcRecord.from(doc.getFields())));
                marcXchangeWriter.endCollection();
                marcXchangeWriter.endDocument();
            }
        }
    }
}
