package org.xbib.interlibrary.catalog.elasticsearch.jsonlines;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.util.Map;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.common.xcontent.XContentType;
import org.xbib.datastructures.json.tiny.Json;
import org.xbib.interlibrary.api.action.worker.WorkerRequest;
import org.xbib.interlibrary.catalog.builder.ConverterInput;
import org.xbib.interlibrary.catalog.builder.StringWorkerRequest;
import org.xbib.interlibrary.catalog.elasticsearch.ElasticsearchTool;
import org.xbib.settings.Settings;

public class JsonLines extends ElasticsearchTool<String> {

    public JsonLines() {
    }

    @Override
    public JsonLinesWorkerPool newPool(Settings settings) throws IOException {
        return new JsonLinesWorkerPool(settings);
    }

    public class JsonLinesWorkerPool extends ElasticsearchWorkerPool<String> {

        public JsonLinesWorkerPool(Settings settings) throws IOException {
            super("json-lines", settings);
        }

        @Override
        public JsonLinesWorker newWorker(String name, int count) throws IOException {
            return new JsonLinesWorker(settings, this);
        }

        @Override
        public WorkerRequest<String> newWorkerRequest(Object input, String origin) {
            return input != null ? new StringWorkerRequest(input.toString(), origin) : null;
        }
    }

    public class JsonLinesWorker extends ElasticsearchWorker<String, JsonLinesWorkerPool> {

        private final JsonLinesWorkerPool pool;

        public JsonLinesWorker(Settings settings, JsonLinesWorkerPool pool) {
            super(settings);
            this.pool = pool;
        }

        @Override
        protected void doExecute(String request) throws IOException {
            try (InputStream in = ConverterInput.getInputStream(request, settings);
                 BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(in, StandardCharsets.UTF_8))) {
                while (true) {
                    String coordinates = bufferedReader.readLine();
                    if (coordinates == null) {
                        break;
                    }
                    String content = bufferedReader.readLine();
                    if (content == null) {
                        break;
                    }
                    Map<String, Object> map = Json.toMap(coordinates);
                    pool.bulkIndex(new IndexRequest().index((String) map.get("_index"))
                            .id((String) map.get("_id")).create(false).source(content, XContentType.JSON));
                }
            }
        }
    }
}
