package org.xbib.interlibrary.catalog.elasticsearch.lvi.config;

import java.io.IOException;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.stream.Stream;
import org.elasticsearch.common.unit.TimeValue;
import org.xbib.elx.api.SearchClient;
import org.xbib.elx.api.SearchDocument;
import org.xbib.interlibrary.api.library.DefaultIndexLibrary;
import org.xbib.interlibrary.api.product.DefaultProduct;
import org.xbib.interlibrary.api.library.IndexLibraryHelper;

public class ElasticsearchConfiguration extends DefaultConfiguration {

    private static final Logger logger = Logger.getLogger(ElasticsearchConfiguration.class.getName());

    private final IndexLibraryHelper indexLibraryHelper;

    private final AtomicBoolean closed;

    private final SearchClient searchClient;

    public ElasticsearchConfiguration(ConfigurationArguments arguments,
                                      SearchClient searchClient) throws IOException {
        super(arguments);
        this.indexLibraryHelper = new IndexLibraryHelper();
        this.searchClient = searchClient;
        this.closed = new AtomicBoolean(false);
        // now we iterate over all entries in the bib index and process each one. This may take some time.
        logger.log(Level.INFO, "processing all library definitions from index");
        Stream<SearchDocument> searchDocumentStream = searchClient.search(srb -> srb
                .setIndices("bib"), TimeValue.timeValueMinutes(1), 1000);
        searchDocumentStream.forEach(this::process);
        logger.log(Level.INFO, "library definitions processed from index, now adding custom library definitions");
        super.addCustomLibraries();
    }

    @Override
    public void close() throws IOException {
        if (closed.compareAndSet(false, true)) {
            super.close();
            searchClient.close();
        }
    }

    private void process(SearchDocument searchDocument) {
        DefaultIndexLibrary.Builder builder = DefaultIndexLibrary.builder();
        indexLibraryHelper.parseLibrary(builder, searchDocument.getFields());
        DefaultIndexLibrary library = builder.build();
        String isil = library.getISIL();
        if (library.getDomain() != null) {
            addLibrary(isil, builder);
        } else {
            addProduct(isil, new DefaultProduct(library.getISIL(), library.getName()));
        }
    }
}
