package org.xbib.interlibrary.catalog.elasticsearch.lvi.config;

import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Collections;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.xbib.datastructures.json.tiny.Json;
import org.xbib.datastructures.json.tiny.JsonException;
import org.xbib.datastructures.tiny.TinyMap;

public class ResourceLoader {

    private static final Logger logger = Logger.getLogger(ResourceLoader.class.getName());

    private final ClassLoader classLoader;

    public ResourceLoader() {
        this(ResourceLoader.class.getClassLoader());
    }

    public ResourceLoader(ClassLoader classLoader) {
        this.classLoader = classLoader;
    }

    public Map<String, Object> loadResourceMap(String resource) {
        try (InputStream inputStream1 = classLoader.getResourceAsStream(resource)) {
            if (inputStream1 != null) {
                String string = new String(inputStream1.readAllBytes(), StandardCharsets.UTF_8);
                return Json.toMap(string);
            }
        } catch (JsonException | IOException e) {
            logger.log(Level.WARNING, e.getMessage() + ", resource = " + resource, e);
        }
        return Collections.emptyMap();
    }

    private Map<String, Object> objectMap(Map<Object, Object> objectMap) {
        TinyMap.Builder<String, Object> map = TinyMap.builder();
        objectMap.forEach((k,v ) -> map.put(k.toString(), v.toString()));
        return map.build();
    }
}
