package org.xbib.interlibrary.catalog.elasticsearch.lvi.config;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class TabSigel {

    private static final Pattern pattern = Pattern.compile("^1\\s(.{21})(.{5}).*");

    private final Map<String, String> map = new HashMap<>();

    public TabSigel() throws IOException {
        load(getClass().getResourceAsStream("tab_sigel.txt"), StandardCharsets.ISO_8859_1);
    }

    public void load(InputStream inputStream, Charset encoding) throws IOException {
        try (BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream, encoding))) {
            bufferedReader.lines().forEach(line -> {
                if (line.trim().length() > 0 && !line.startsWith("!")) {
                    Matcher m = pattern.matcher(line);
                    if (m.matches()) {
                        String sigel = m.group(1).trim();
                        String owner = m.group(2).trim();
                        map.put(owner, sigel);
                    }
                }
            });
        }
    }

    public Map<String, String> getMap() {
        return map;
    }

    public TabSigel add(String key, String value) {
        this.map.put(key, value);
        return this;
    }

    public TabSigel add(Map<String, Object> map) {
        if (map == null) {
            return this;
        }
        for (Map.Entry<String, Object> entry : map.entrySet()) {
            this.map.put(entry.getKey(), entry.getValue().toString());
        }
        return this;
    }

    public boolean containsKey(String key) {
        return map.containsKey(key);
    }

    public String get(String value) {
        return map.getOrDefault(value, value);
    }
}
