package org.xbib.interlibrary.catalog.elasticsearch.lvi.record;

import org.xbib.interlibrary.api.library.Material;
import org.xbib.interlibrary.catalog.elasticsearch.lvi.Service;
import org.xbib.interlibrary.catalog.elasticsearch.lvi.config.Configuration;
import org.xbib.interlibrary.catalog.elasticsearch.lvi.service.DefaultService;
import org.xbib.marc.MarcField;

public final class HbzAlephHoldings extends AbstractHoldings {

    private HbzAlephHoldings(Builder builder) {
        super();
        this.builder = builder;
    }

    public static Builder builder(Configuration configuration,
                                  HbzAlephRecord hbzAlephRecord,
                                  MarcField marcField) {
        return new Builder(configuration, hbzAlephRecord, marcField);
    }

    @Override
    protected String getIndicator() {
        return builder.material == Material.ONLINE_RESOURCE ? "1 " : "0 ";
    }

    @Override
    protected void customSubfields(MarcField.Builder marcFieldBuilder) {
    }

    public static class Builder extends AbstractHoldings.Builder {

        protected Builder(Configuration configuration,
                          HbzAlephRecord hbzAlephRecord,
                          MarcField marcField) {
            super(configuration, hbzAlephRecord, null);
            setISIL(configuration, marcField);
            addAlias(marcField.getFirstSubfieldValue("a"));
            Service service = new DefaultService();
            service.setMaterial(material);
            if (fullIsil != null) {
                service.setISIL(fullIsil);
                setIdentifier("(" + fullIsil + ")" + hbzAlephRecord.getIdentifier());
            } else {
                service.setISIL(isil);
                setIdentifier("(" + isil + ")" + hbzAlephRecord.getIdentifier());
            }
            String collection = marcField.getFirstSubfieldValue("b");
            service.setCollection(collection);
            // Example: "F_EF" should map service to DE-290-EFB
            setSubLibrary(service, collection);
            String callnumber = marcField.getFirstSubfieldValue("c");
            service.setCallnumber(callnumber);
            // for serials, "d" is used for textual holdings
            if (hbzAlephRecord.isSerialMirror()) {
                String textualHoldings = marcField.getFirstSubfieldValue("d");
                service.setTextualHoldings(textualHoldings);
                // we must delete mirror serial. Why?
                // It is impossible to match "prefix" + "textual holdings" with ZDB MARC data.
                // Example: check the 3 services of DE-385 in "Nature : the international journal of science" (ZDB 120714-3)
                setDeleted(true);
            } else {
                // otherwise for description
                String description = marcField.getFirstSubfieldValue("d");
                service.setDescription(description);
            }
            String statusText = marcField.getFirstSubfieldValue("e");
            setStatusCode(service, statusText, "", "");
            // Aleph online monograph "catalog enrichment" is stored in title record 655 $e9
            if (material == Material.ONLINE_RESOURCE) {
                if (record.getChronology() != null) {
                    service.setTextualHoldings(record.getChronology());
                } else {
                    if (record.getPublicationDate() != null) {
                        service.setTextualHoldings(record.getPublicationDate().toString());
                    }
                }
            } else {
                if (service.getTextualHoldings() == null) {
                    if (record.getPublicationDate() != null) {
                        service.setTextualHoldings(record.getPublicationDate().toString());
                    }
                }
            }
            // we skip all Aleph online resources without textual holdings because we have only the ISIL and we
            // expect full data from EZB licenses.
            if (material != Material.ONLINE_RESOURCE || service.getTextualHoldings() != null) {
                service.computeYears();
                services.add(service);
            }
        }

        @Override
        public HbzAlephHoldings build() {
            return new HbzAlephHoldings(this);
        }
    }
}
