package org.xbib.interlibrary.catalog.elasticsearch.lvi.record;

import java.util.Objects;
import org.xbib.interlibrary.api.library.IndexLibrary;
import org.xbib.interlibrary.catalog.elasticsearch.lvi.Service;
import org.xbib.interlibrary.catalog.elasticsearch.lvi.config.Configuration;
import org.xbib.interlibrary.catalog.elasticsearch.lvi.service.DefaultService;
import org.xbib.marc.MarcField;
import org.xbib.marc.MarcRecord;

public final class HbzAlmaHoldings extends AbstractHoldings {

    private HbzAlmaHoldings(Builder builder) {
        super();
        this.builder = builder;
    }

    public static Builder builder(Configuration configuration,
                                  HbzAlmaRecord hbzAlmaRecord,
                                  MarcField marcField) {
        return new Builder(configuration, hbzAlmaRecord, marcField);
    }

    @Override
    protected String getIndicator() {
        return "0 ";
    }

    @Override
    protected void customSubfields(MarcField.Builder marcFieldBuilder) {
    }

    public static class Builder extends AbstractHoldings.Builder {

        protected Builder(Configuration configuration,
                          HbzAlmaRecord hbzAlmaRecord,
                          MarcField marcField) {
            super(configuration, hbzAlmaRecord, marcField);
            Service service = new DefaultService();
            service.setMaterial(material);
            if (fullIsil != null) {
                service.setISIL(fullIsil);
            } else {
                service.setISIL(isil);
            }
            services.add(service);
        }

        @Override
        protected String findISILCandidate(MarcRecord marcRecord, MarcField marcfield) {
            String code = marcfield.getLastSubfieldValue("M");
            // special code for libraries without cataloging
            // "49" Exlibris code for Germany, "HBZ" Hochschulbibliothekszentrum, "FIZ" = "frozen inventory zone"
            if ("49HBZ_FIZ".contentEquals(code) || "49HBZ_BRIDGE_FIZ".contentEquals(code)) {
                // u = Aleph Owner, take precedence over "M"
                String u = marcfield.getLastSubfieldValue("u");
                if (u == null) {
                    // try H52 $b to take precedence over "M"
                    StringBuilder sb = new StringBuilder();
                    marcRecord.getAll("H52").forEach(mf -> {
                        String b = mf.getLastSubfieldValue("b");
                        if (b != null) {
                            sb.append(b);
                        }
                    });
                    if (!sb.isEmpty()) {
                        return sb.toString();
                    }
                } else {
                    return u;
                }
            }
            return code;
        }

        @Override
        protected IndexLibrary fixLibrary(IndexLibrary library,
                                          MarcRecord marcRecord,
                                          MarcField marcfield) {
            String link = marcfield.getLastSubfieldValue("8");
            if (link == null) {
                return library;
            }
            // find H52 field by link
            StringBuilder sb = new StringBuilder();
            marcRecord.getAll("H52").forEach(mf -> {
                String hLink = mf.getLastSubfieldValue("8");
                if (link.equals(hLink)) {
                    String code = mf.getFirstSubfieldValue("b");
                    sb.append(code);
                }
            });
            String extraAlmaCode = sb.toString();
            if (!extraAlmaCode.isEmpty()) {
                // we must have a custom ISIL mapping configured in "sublibraries"
                if (library.getSubLibraries() != null) {
                    if (library.getSubLibraries().containsKey(extraAlmaCode)) {
                        IndexLibrary extraLibrary = configuration.getLibraries().get(library.getSubLibraries().get(extraAlmaCode).toString());
                        if (extraLibrary != null) {
                            return extraLibrary;
                        }
                    }
                }
                IndexLibrary extraLibrary = configuration.getLibraries().get(extraAlmaCode);
                return Objects.requireNonNullElse(extraLibrary, library);
            }
            return library;
        }

        @Override
        public HbzAlmaHoldings build() {
            return new HbzAlmaHoldings(this);
        }
    }
}
