package org.xbib.interlibrary.catalog.elasticsearch.lvi.record;

import org.xbib.datastructures.common.LinkedHashSetMultiMap;
import org.xbib.datastructures.common.MultiMap;
import org.xbib.interlibrary.api.library.IndexLibrary;
import org.xbib.interlibrary.api.library.Material;
import org.xbib.interlibrary.catalog.elasticsearch.lvi.Service;
import org.xbib.interlibrary.catalog.elasticsearch.lvi.config.Configuration;
import org.xbib.interlibrary.catalog.elasticsearch.lvi.service.DefaultService;
import org.xbib.marc.MarcField;
import org.xbib.marc.MarcRecord;

import java.util.Objects;

public final class HbzAlmaOnlineResource extends AbstractHoldings {

    private static final MultiMap<String, String> unknownSublibraries = new LinkedHashSetMultiMap<>();

    private HbzAlmaOnlineResource(Builder builder) {
        super();
        this.builder = builder;
    }

    public static MultiMap<String, String> getUnknownSublibraries() {
        return unknownSublibraries;
    }

    public static Builder builder(Configuration configuration,
                                  HbzAlmaRecord hbzAlmaRecord,
                                  MarcField marcField) {
        return new Builder(configuration, hbzAlmaRecord, marcField);
    }

    @Override
    protected String getIndicator() {
        return "1 ";
    }

    @Override
    protected void customSubfields(MarcField.Builder marcFieldBuilder) {
    }

    public static class Builder extends AbstractHoldings.Builder {

        protected Builder(Configuration configuration,
                          HbzAlmaRecord hbzAlmaRecord,
                          MarcField marcField) {
            super(configuration, hbzAlmaRecord, marcField);
            String enabled = marcField.getFirstSubfieldValue("b");
            if ("Available".equals(enabled)) {
                DefaultService service = new DefaultService();
                service.setMaterial(Material.ONLINE_RESOURCE);
                String productName = marcField.getFirstSubfieldValue("q");
                String almaLibraryCode = marcField.getLastSubfieldValue("M");
                String almaDeliveryUrl = marcField.getFirstSubfieldValue("d");
                if (almaDeliveryUrl != null) {
                    Service.LabeledUrl labeledUrl = new Service.LabeledUrl();
                    labeledUrl.setUrl(almaDeliveryUrl.replace("49HBZ_NETWORK", almaLibraryCode));
                    labeledUrl.setLabel(productName);
                    service.addUrl(labeledUrl);
                }
                // her we set the default value "b" (Kopie)
                service.setFirstStatusCode("b");
                // Alma portfolio (POR) does not have textual holdings, so we use record publication date if present
                if (service.getTextualHoldings() == null) {
                    if (record.getPublicationDate() != null) {
                        service.setTextualHoldings(record.getPublicationDate().toString());
                    }
                }
                service.computeYears();
                services.add(service);
                products.addAll(hbzAlmaRecord.getProducts()); // Produkt-ISIL
            }
        }

        @Override
        protected String findISILCandidate(MarcRecord marcRecord, MarcField marcfield) {
            return marcfield.getLastSubfieldValue("M");
        }

        @Override
        protected IndexLibrary fixLibrary(IndexLibrary library,
                                          MarcRecord marcRecord,
                                          MarcField marcfield) {
            String extraAlmaCode = marcfield.getLastSubfieldValue("u");
            if (extraAlmaCode == null) {
                return library;
            }
            // we must have a custom ISIL mapping configured in "sublibraries"
            if (library.getSubLibraries() != null) {
                if (library.getSubLibraries().containsKey(extraAlmaCode)) {
                    IndexLibrary extraLibrary = configuration.getLibraries().get(library.getSubLibraries().get(extraAlmaCode).toString());
                    if (extraLibrary != null) {
                        return extraLibrary;
                    }
                } else {
                    unknownSublibraries.put(library.getISIL(), extraAlmaCode);
                }
            }
            IndexLibrary extraLibrary = configuration.getLibraries().get(extraAlmaCode);
            return Objects.requireNonNullElse(extraLibrary, library);
        }

        @Override
        public HbzAlmaOnlineResource build() {
            return new HbzAlmaOnlineResource(this);
        }
    }
}
