package org.xbib.interlibrary.catalog.elasticsearch.lvi.record;

import org.xbib.interlibrary.catalog.elasticsearch.lvi.Service;
import org.xbib.interlibrary.catalog.elasticsearch.lvi.config.Configuration;
import org.xbib.interlibrary.catalog.elasticsearch.lvi.service.DefaultService;
import org.xbib.marc.MarcField;
import org.xbib.marc.MarcRecord;

public final class HbzGviHoldings extends AbstractHoldings {

    private HbzGviHoldings(Builder builder) {
        super();
        this.builder = builder;
    }

    public static Builder builder(Configuration configuration,
                                  HbzGviRecord hbzGviRecord,
                                  MarcField marcField) {
        return new Builder(configuration, hbzGviRecord, marcField);
    }

    @Override
    protected String getIndicator() {
        return builder.marcField.getIndicator();
    }

    @Override
    protected void customSubfields(MarcField.Builder marcFieldBuilder) {
    }

    public static class Builder extends AbstractHoldings.Builder {

        protected Builder(Configuration configuration,
                          HbzGviRecord hbzGviRecord,
                          MarcField marcField) {
            super(configuration, hbzGviRecord, marcField);
            Service service = new DefaultService();
            service.setMaterial(material);
            if (fullIsil != null) {
                service.setISIL(fullIsil);
            } else {
                service.setISIL(isil);
            }
            String collection = marcField.getFirstSubfieldValue("h");
            service.setCollection(collection);
            setSubLibrary(service, collection);
            String callnumber = marcField.getFirstSubfieldValue("g");
            service.setCallnumber(callnumber);
            String description = marcField.getFirstSubfieldValue("j");
            service.setDescription(description);
            String statusText = marcField.getFirstSubfieldValue("d");
            setStatusCode(service, statusText, "", "");
            if (service.getTextualHoldings() == null) {
                // monograph publication date
                if (record.getPublicationDate() != null) {
                    service.setTextualHoldings(record.getPublicationDate().toString());
                }
            }
            service.computeYears();
            services.add(service);
        }

        @Override
        protected String findISILCandidate(MarcRecord marcRecord, MarcField marcfield) {
            return marcfield.getLastSubfieldValue("b");
        }

        @Override
        public HbzGviHoldings build() {
            return new HbzGviHoldings(this);
        }
    }
}
