package org.xbib.interlibrary.catalog.elasticsearch.lvi.record;

import java.util.Objects;
import java.util.Optional;
import java.util.function.Consumer;
import java.util.stream.Stream;
import org.xbib.interlibrary.catalog.elasticsearch.lvi.config.Configuration;
import org.xbib.marc.MarcField;
import org.xbib.marc.MarcRecord;

public class HbzGviRecord extends AbstractRecord {

    protected HbzGviRecord(Builder builder) {
        super(builder);
    }

    public static Builder builder(Configuration configuration,
                                  MarcRecord marcRecord) {
        return new Builder(configuration, marcRecord);
    }

    public void handleResources(Consumer<MarcField> consumer) {
        getMarcRecord().all("924", consumer::accept);
    }

    public static class Builder extends AbstractRecord.Builder {

        protected Builder(Configuration configuration,
                          MarcRecord marcRecord) {
            super(configuration, "hbzgvi", marcRecord);
        }

        @Override
        public HbzGviRecord build() {
            return new HbzGviRecord(this);
        }

        @Override
        protected void findDeleted() {
            super.findDeleted();
            // lookup ZDB-ID from MARC 016, if present, we have a ZDB record. and we want to ignore it. Set this record to deleted.
            Stream<String> stream = marcRecord.getAll("016").stream()
                    .filter(mf -> "DE-600".equals(mf.getFirstSubfieldValue("2")))
                    .map(mf -> mf.getFirstSubfieldValue("a"));
            Optional<String> optionalString = stream.filter(Objects::nonNull).findFirst();
            optionalString.ifPresent((zdbId) -> this.isDeleted = true);
        }

        @Override
        protected void findIdentifier() {
            MarcField marcField = marcRecord.getFirst("001");
            if (marcField != null) {
                this.identifier = marcField.getFirstSubfieldValue("_");
                marcField = marcRecord.getFirst("003");
                if (marcField != null) {
                    this.identifier = "(" + marcField.getFirstSubfieldValue("_") + ")" + identifier;
                }
            }
            this.externalIdentifier = this.identifier;
        }
    }
}
