package org.xbib.interlibrary.catalog.elasticsearch.lvi.record;

import org.xbib.interlibrary.catalog.elasticsearch.lvi.Service;
import org.xbib.interlibrary.catalog.elasticsearch.lvi.config.Configuration;
import org.xbib.interlibrary.catalog.elasticsearch.lvi.service.DefaultService;
import org.xbib.marc.MarcField;
import org.xbib.marc.MarcRecord;

public final class HbzfixHoldings extends AbstractHoldings {

    private HbzfixHoldings(Builder builder) {
        super();
        this.builder = builder;
    }

    public static Builder builder(Configuration configuration,
                                  HbzfixRecord hbzfixRecord,
                                  MarcField marcField) {
        return new Builder(configuration, hbzfixRecord, marcField);
    }

    @Override
    protected String getIndicator() {
        return "0 ";
    }

    @Override
    protected void customSubfields(MarcField.Builder marcFieldBuilder) {
    }

    public static class Builder extends AbstractHoldings.Builder {

        protected Builder(Configuration configuration,
                          HbzfixRecord hbzfixRecord,
                          MarcField marcField) {
            super(configuration, hbzfixRecord, marcField);
            Service service = new DefaultService();
            service.setMaterial(material);
            String collection = marcField.getFirstSubfieldValue("h");
            service.setCollection(collection);
            setSubLibrary(service, collection);
            String callnumber = marcField.getFirstSubfieldValue("g");
            service.setCallnumber(callnumber);
            String description = marcField.getFirstSubfieldValue("j");
            service.setDescription(description);
            String statusText = marcField.getFirstSubfieldValue("d");
            setStatusCode(service, statusText, "", "");
            // if no status set, use default value "c"
            if (service.getFirstStatusCode() == null) {
                service.setFirstStatusCode("c");
            }
            service.computeYears();
            services.add(service);
        }

        @Override
        protected String findISILCandidate(MarcRecord marcRecord, MarcField marcfield) {
            return marcfield.getLastSubfieldValue("b");
        }

        @Override
        public HbzfixHoldings build() {
            return new HbzfixHoldings(this);
        }
    }
}
