package org.xbib.interlibrary.catalog.elasticsearch.lvi.record;

import java.util.function.Consumer;
import org.xbib.interlibrary.catalog.elasticsearch.lvi.config.Configuration;
import org.xbib.marc.MarcField;
import org.xbib.marc.MarcRecord;

public class HbzfixRecord extends AbstractRecord {

    protected HbzfixRecord(Builder builder) {
        super(builder);
    }

    public static Builder builder(Configuration configuration,
                                  MarcRecord marcRecord) {
        return new Builder(configuration, marcRecord);
    }

    public void handleResources(Consumer<MarcField> consumer) {
        getMarcRecord().all("924", consumer::accept);
    }

    public static class Builder extends AbstractRecord.Builder {

        protected Builder(Configuration configuration,
                          MarcRecord marcRecord) {
            super(configuration, "hbzfix", marcRecord);
        }

        @Override
        public HbzfixRecord build() {
            return new HbzfixRecord(this);
        }

        @Override
        protected void findDeleted() {
            super.findDeleted();
            MarcField marcField = marcRecord.getFirst("001");
            // data is unreliable, even 001 might be missing
            if (marcField == null) {
                this.isDeleted = true;
            }
        }

        @Override
        protected void findIdentifier() {
            MarcField marcField = marcRecord.getFirst("001");
            // the identifier is already merged with 003 so do not look for 003
            this.identifier = marcField.getFirstSubfieldValue("_");
            this.externalIdentifier = this.identifier;
            // drop records with broken identifiers
            if (identifier != null && identifier.length() > 64) {
                this.isDeleted = true;
                return;
            }
            // if no MARC 008 or 245, drop record
            if (marcRecord.getFirst("008") == null || marcRecord.getFirst("245") == null) {
                this.isDeleted = true;
            }
        }
    }
}
