package org.xbib.interlibrary.catalog.elasticsearch.lvi.service;

import java.time.Clock;
import java.time.LocalDate;
import java.time.Year;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Objects;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.xbib.interlibrary.api.library.Material;
import org.xbib.interlibrary.api.library.Mode;
import org.xbib.interlibrary.catalog.elasticsearch.lvi.Service;
import org.xbib.interlibrary.catalog.matching.chronology.EnumerationAndChronologyHelper;

public class DefaultService implements Service, Comparable<Service> {

    private Material material;

    private Mode mode;

    private String isil;

    private String collection;

    private String callnumber;

    private String callnumber2;

    private String description;

    private String textualholdings;

    private int delta;

    private final LabeledUrlList urls;

    /**
     * Computed from textual holdings
     */
    private Collection<Integer> years;

    private String movingWall;

    /**
     * Interlibrary loan indicator.
     */
    private String firstStatusCode;

    /**
     * Interlibrary restriction codes.
     */
    private String secondStatusCode;

    /**
     * Comment on interlibrary loan indicator.
     */
    private String thirdStatusCode;

    /**
     * Item process code is an Alma specfic entity, an extra information
     * to qualify items inplace / not in place.
     */
    private String itemProcessCode;

    public DefaultService() {
        this.urls = new LabeledUrlList();
    }

    @Override
    public void setMaterial(Material material) {
        this.material = material;
    }

    @Override
    public Material getMaterial() {
        return material;
    }

    @Override
    public void setMode(Mode mode) {
        this.mode = mode;
    }

    @Override
    public Mode getMode() {
        return mode;
    }

    @Override
    public void setISIL(String isil) {
        this.isil = isil;
    }

    @Override
    public String getISIL() {
        return isil;
    }

    @Override
    public void setCollection(String collection) {
        this.collection = collection;
    }

    @Override
    public String getCollection() {
        return collection;
    }

    @Override
    public void setCallnumber(String callnumber) {
        this.callnumber = callnumber;
    }

    @Override
    public String getCallnumber() {
        return callnumber;
    }

    @Override
    public void setCallnumber2(String callnumber2) {
        this.callnumber2 = callnumber2;
    }

    public String getCallnumber2() {
        return callnumber2;
    }

    @Override
    public void setDescription(String description) {
        this.description = description;
    }

    @Override
    public String getDescription() {
        StringBuilder sb = new StringBuilder();
        if (description != null) {
            sb.append(description);
        }
        if (delta > 0) {
            if (!sb.isEmpty()) {
                sb.append("; ");
            }
            if (delta == 1) {
                sb.append("nur der letzte Jahrgang ist zugänglich");
            } else {
                sb.append("nur die letzten ").append(delta).append(" Jahrgänge sind zugänglich");
            }
        }
        if (delta < 0) {
            if (!sb.isEmpty()) {
                sb.append("; ");
            }
            if (delta == -1) {
                sb.append("der letzte Jahrgang ist nicht zugänglich");
            } else {
                sb.append("die letzten ").append(-delta).append(" Jahrgänge sind nicht zugänglich");
            }
        }
        return sb.isEmpty() ? null : sb.toString();
    }

    @Override
    public void setDelta(int delta) {
        this.delta = delta;
    }

    @Override
    public int getDelta() {
        return delta;
    }

    @Override
    public void addUrl(LabeledUrl url) {
        urls.add(url);
    }

    @Override
    public LabeledUrlList getUrls() {
        return urls;
    }

    @Override
    public void setTextualHoldings(String textualholdings) {
        this.textualholdings = textualholdings;
    }

    @Override
    public String getTextualHoldings() {
        return textualholdings;
    }

    @Override
    public void setMovingWall(String movingWall) {
        this.movingWall = movingWall;
    }

    @Override
    public String getMovingWall() {
        return movingWall;
    }

    @Override
    public void setYears(Collection<Integer> years) {
        this.years = years;
    }

    @Override
    public Collection<Integer> getYears() {
        return years;
    }

    @Override
    public void setFirstStatusCode(String firstStatusCode) {
        this.firstStatusCode = firstStatusCode;
    }

    @Override
    public String getFirstStatusCode() {
        return firstStatusCode;
    }

    @Override
    public void setSecondStatusCode(String secondStatusCode) {
        this.secondStatusCode = secondStatusCode;
    }

    @Override
    public String getSecondStatusCode() {
        return secondStatusCode;
    }

    @Override
    public void setThirdStatusCode(String thirdStatusCode) {
        this.thirdStatusCode = thirdStatusCode;
    }

    @Override
    public String getThirdStatusCode() {
        return thirdStatusCode;
    }

    @Override
    public void setItemProcessCode(String itemProcessCode) {
        this.itemProcessCode = itemProcessCode;
    }

    @Override
    public String getItemProcessCode() {
        return itemProcessCode;
    }

    @Override
    public String toString() {
        return "[" + isil + "][" + collection + "][" + callnumber + "][" + textualholdings + "]";
    }

    @Override
    public int hashCode() {
        return Objects.hash(isil, collection, callnumber, textualholdings);
    }

    @Override
    public boolean equals(Object object) {
        return object instanceof DefaultService && hashCode() == object.hashCode();
    }

    @Override
    public int compareTo(Service service) {
        return toString().compareTo(service.toString());
    }

    @Override
    public void computeYears() {
        EnumerationAndChronologyHelper eac = new EnumerationAndChronologyHelper();
        Collection<Integer> years = eac.getDates(textualholdings);
        // hack: publishers use new year for publications very early
        if (MAXIMUM_PUBLICATION_DATE > CURRENT_DATE) {
            int extraYear = getMovingWall() == null ? MAXIMUM_PUBLICATION_DATE : CURRENT_DATE;
            // fix all open end years for maximum publication date (6 months ahead)
            for (int i = 0; i < eac.getBegin().size(); i++) {
                if (eac.getOpen().get(i) != null && eac.getOpen().get(i)) {
                    years.add(extraYear);
                }
            }
        }
        if (movingWall != null) {
            for (MovingWall movingWall : computeMovingWalls(eac)) {
                if (movingWall.last != null && movingWall.back != null) {
                    for (int d = movingWall.back; d <= movingWall.last; d++) {
                        if (movingWall.add) {
                            years.add(d);
                        } else {
                            years.remove(d);
                        }
                    }
                }
            }
        }
        this.years = years;
    }

    private List<MovingWall> computeMovingWalls(EnumerationAndChronologyHelper eac) {
        String movingWallPattern = getMovingWall();
        // we may have several moving walls but in reality there is only one
        List<MovingWall> movingWalls = new ArrayList<>();
        if (!eac.getEnd().isEmpty()) {
            if (movingWallPattern != null && !movingWallPattern.isEmpty()) {
                Integer delta = null;
                Matcher m = movingWallYearPattern.matcher(movingWallPattern);
                if (m.find()) {
                    delta = Integer.parseInt(m.group(1));
                } else {
                    m = movingWallMonthPattern.matcher(movingWallPattern);
                    if (m.find()) {
                        delta = Integer.parseInt(m.group(1)) / 12;
                    }
                }
                if (delta != null) {
                    MovingWall movingWall = new MovingWall();
                    movingWall.last = eac.getEnd().getLast() != null ? eac.getEnd().getLast() : CURRENT_DATE;
                    movingWall.back = movingWall.last - delta + 1;
                    if (movingWallPattern.startsWith("+")) {
                        movingWall.add = true;
                        setDelta(delta);
                    } else if (movingWallPattern.startsWith("-")) {
                        movingWall.add = false;
                        setDelta(-delta);
                    }
                    movingWalls.add(movingWall);
                }
            }
        }
        return movingWalls;
    }

    private static class MovingWall {
        Integer last;
        Integer back;
        boolean add;
    }

    /*
     * The current year.
     */
    private static final Integer CURRENT_DATE =
            Year.of(LocalDate.now(Clock.systemDefaultZone()).getYear()).getValue();


    /*
     * Respect publication dates of six months ahead.
     */
    private static final Integer MAXIMUM_PUBLICATION_DATE =
            Year.of(LocalDate.now(Clock.systemDefaultZone()).plusMonths(6).getYear()).getValue();


    private static final Pattern movingWallMonthPattern = Pattern.compile("^[+-](\\d+)M$");

    private static final Pattern movingWallYearPattern = Pattern.compile("^[+-](\\d+)Y$");

}
