package org.xbib.interlibrary.catalog.elasticsearch.lvi.statistics;

import java.io.IOException;
import java.io.Writer;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Map;
import java.util.stream.Collectors;
import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.marc.json.JsonBuilder;
import org.xbib.settings.Settings;
import static java.util.Map.Entry;

public abstract class CodeMapStatistics<C> implements Statistics<C, Statistics.CodeMap> {

    private final Settings settings;

    protected final boolean enabled;

    protected final TinyMap.Builder<String, CodeMap> codeMapBuilder;

    private Map<String, CodeMap> map;

    protected boolean closed;

    public CodeMapStatistics(Settings settings, String prefix, String type) {
        this.settings = settings;
        this.enabled = settings.getAsBoolean("statistics." + prefix + "." + type + ".enabled", false);
        this.codeMapBuilder = TinyMap.builder();
    }

    public abstract void count(C countable);

    public Map<String, Integer> getSummary() {
        if (map == null) {
            map = codeMapBuilder.build();
            closed = true;
        }
        return map
                .values()
                .stream()
                .map(CodeMap::map)
                .flatMap(m -> m.entrySet().stream())
                .collect(Collectors.groupingBy(Entry::getKey, Collectors.summingInt(Entry::getValue)));
    }

    @Override
    public Map<String, CodeMap> map() {
        if (map == null) {
            map = codeMapBuilder.build();
            closed = true;
        }
        return map;
    }

    public Map<String, Object> objectMap() {
        TinyMap.Builder<String, Object> map = TinyMap.builder();
        map().forEach((k, v) -> map.put(k, v.map()));
        return map.build();
    }

    @Override
    public void write(String fileName) throws IOException {
        if (enabled && settings.containsSetting("statistics.path")) {
            String statisticsPath = settings.get("statistics.path");
            Files.createDirectories(Paths.get(statisticsPath));
            try (Writer writer = Files.newBufferedWriter(Paths.get(statisticsPath, fileName))) {
                JsonBuilder builder = JsonBuilder.builder();
                builder.buildMap(objectMap());
                String string = builder.build();
                writer.write(string);
            }
        }
    }
}
