package org.xbib.interlibrary.catalog.elasticsearch.lvi.statistics;

import java.io.IOException;
import java.io.Writer;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Map;
import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.marc.json.JsonBuilder;
import org.xbib.settings.Settings;

public abstract class CountStatistics<C> implements Statistics<C, Integer> {

    private final Settings settings;

    protected final boolean enabled;

    protected final TinyMap.Builder<String, Integer> integerMapBuilder;

    private Map<String, Integer> map;

    protected boolean closed;

    public CountStatistics(Settings settings, String prefix, String type) {
        this.settings = settings;
        this.enabled = settings.getAsBoolean("statistics." + prefix + "." + type + ".enabled", false);
        this.integerMapBuilder = TinyMap.builder();
    }

    @Override
    public Map<String, Integer> map() {
        if (map == null) {
            map = integerMapBuilder.build();
            closed = true;
        }
        return map;
    }

    public Map<String, Object> objectMap() {
        TinyMap.Builder<String, Object> map = TinyMap.builder();
        map.putAll(map());
        return map.build();
    }

    public Integer summary() {
        return map().values().stream().reduce(0, Integer::sum);
    }

    @Override
    public void write(String fileName) throws IOException {
        if (enabled && settings.containsSetting("statistics.path")) {
            String statisticsPath = settings.get("statistics.path");
            Files.createDirectories(Paths.get(statisticsPath));
            try (Writer writer = Files.newBufferedWriter(Paths.get(statisticsPath, fileName))) {
                JsonBuilder builder = JsonBuilder.builder();
                builder.buildMap(objectMap());
                String string = builder.build();
                writer.write(string);
            }
        }
    }
}
