package org.xbib.interlibrary.catalog.elasticsearch.lvi.statistics;

import java.util.ConcurrentModificationException;
import org.xbib.interlibrary.catalog.elasticsearch.lvi.Holdings;
import org.xbib.interlibrary.catalog.elasticsearch.lvi.Service;
import org.xbib.settings.Settings;

public class ResourceCollectionStatistics extends CodeMapStatistics<Holdings> {

    public ResourceCollectionStatistics(Settings settings, String prefix) {
        super(settings, prefix, "resources.collection");
    }

    @Override
    public void count(Holdings holdings) {
        if (!enabled) {
            return;
        }
        if (closed) {
            throw new ConcurrentModificationException();
        }
        if (holdings == null) {
            return;
        }
        String category = holdings.getCategory();
        if (category == null) {
            return;
        }
        if (isCollectionFeasible(holdings)) {
            CodeMap codeMap = codeMapBuilder.getOrDefault(category, new CodeMap());
            for (Service service : holdings.getServices()) {
                String collection = service.getCollection();
                if (collection != null) {
                    codeMap.add(clean(collection));
                }
            }
            codeMapBuilder.put(category, codeMap);
        }
    }

    private boolean isCollectionFeasible(Holdings holdings) {
        // save memory, we do not want to count outside our domain "NRW"
        if (!"NRW".equals(holdings.getDomain())) {
            return false;
        }
        String isil = holdings.getISIL();
        // skip "bad" status and location ISILs. These ISIL store free text in the collection fields.
        return !"DE-52".equals(isil) &&
                !"DE-131".equals(isil) &&
                !"DE-211".equals(isil) &&
                !"DE-242".equals(isil) &&
                !"DE-260".equals(isil) &&
                !"DE-278".equals(isil) &&
                !"DE-Gk1".equals(isil) &&
                !"DE-X1".equals(isil) &&
                !"DE-Jhn1".equals(isil) &&
                !"DE-Kn38".equals(isil) &&
                !"DE-Kow1".equals(isil) &&
                !"DE-Hen2".equals(isil) &&
                !"DE-B23".equals(isil);
    }

    private static String clean(String collection) {
        // there may be formats like "<license>: <years>", so we cut the part after colon
        int pos = collection.indexOf(':');
        if (pos > 0 ) {
            return collection.substring(0, pos);
        }
        // there may be formats like "<license> <br/> <years>", so we cut the part after the br tag
        pos = collection.indexOf("<br/>");
        if (pos > 0 ) {
            return collection.substring(0, pos);
        }
        return collection;
    }
}
