package org.xbib.interlibrary.catalog.elasticsearch.lvi.statistics;

import java.io.IOException;
import java.text.MessageFormat;
import java.util.Iterator;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.elx.api.IndexDefinition;
import org.xbib.interlibrary.catalog.elasticsearch.ElxOutputHelper;
import org.xbib.interlibrary.catalog.elasticsearch.lvi.Holdings;
import org.xbib.interlibrary.catalog.elasticsearch.lvi.Service;
import org.xbib.marc.json.JsonBuilder;
import org.xbib.settings.Settings;

public class StatusCodeMapStatistics {

    private static final Logger logger = Logger.getLogger(StatusCodeMapStatistics.class.getName());

    private final TinyMap.Builder<String, CodeMap> codeMaps;

    public StatusCodeMapStatistics() {
        this.codeMaps = TinyMap.builder();
    }

    public synchronized void count(Holdings wellDefined, Holdings undefined) {
        Iterator<Service> iterator1 = wellDefined.getServices().iterator();
        Iterator<Service> iterator2 = undefined.getServices().iterator();
        while (iterator1.hasNext() && iterator2.hasNext()) {
            Service service1 = iterator1.next();
            Service service2 = iterator2.next();
            count(service1, service2);
        }
    }

    private void count(Service wellDefined, Service undefined) {
        if (wellDefined.getFirstStatusCode() == null || undefined.getFirstStatusCode() == null) {
            return;
        }
        String isil = wellDefined.getISIL();
        if (isil == null) {
            return;
        }
        CodeMap codeMap = codeMaps.getOrDefault(isil, new CodeMap());
        codeMap.set(undefined.getFirstStatusCode(), wellDefined.getFirstStatusCode());
        codeMaps.put(isil, codeMap);
    }

    public Map<String, CodeMap> getCodeMaps() {
        return codeMaps.build();
    }

    public Map<String, Object> getCodeMaps(String isil) {
        Map<String, CodeMap> map = codeMaps.build();
        return map.containsKey(isil) ? map.get(isil).objectMap() : Map.of();
    }

    public void index(Settings settings,
                       ElxOutputHelper elxOutputHelper,
                       IndexDefinition indexDefinition) throws IOException {
        for (Map.Entry<String, CodeMap> entry : getCodeMaps().entrySet()) {
            String isil = entry.getKey();
            CodeMap codeMap = entry.getValue();
            JsonBuilder builder = JsonBuilder.builder();
            builder.beginMap();
            builder.buildKey("isil")
                    .buildValue(isil);
            builder.buildKey("statuscodes");
            builder.beginMap();
            for (Map.Entry<String, String> e : codeMap.map().entrySet()) {
                builder.buildKey(e.getKey()).buildValue(e.getValue());
            }
            builder.endMap();
            builder.endMap();
            String content = builder.build();
            if (!content.isEmpty()) {
                if (settings.getAsBoolean("enabled", false)) {
                    elxOutputHelper.index(indexDefinition,
                            null, false, content);
                } else {
                    logger.log(Level.FINE, MessageFormat.format("not indexing {0}/{1}/{2} {3}",
                            indexDefinition.getFullIndexName(),
                            indexDefinition.getType(),
                            null, content));
                }
            }
        }
    }

    public static class CodeMap  {

        private final TinyMap.Builder<String, String> builder;

        public CodeMap() {
            this.builder = TinyMap.builder();
        }

        public void set(String code, String value) {
            builder.put(code, value);
        }

        public Map<String, String> map() {
            return builder.build();
        }

        public Map<String, Object> objectMap() {
            TinyMap.Builder<String, Object> map = TinyMap.builder();
            map.putAll(map());
            return map.build();
        }

        @Override
        public String toString() {
            return map().toString();
        }
    }
}
