package org.xbib.interlibrary.catalog.elasticsearch.marc;

import java.io.IOException;
import java.io.InputStream;
import org.xbib.interlibrary.catalog.builder.MarcXchangeJSONLinesReader;
import org.xbib.interlibrary.catalog.entities.CatalogEntityBuilder;
import org.xbib.interlibrary.catalog.entities.CatalogEntityWorkerPool;
import org.xbib.marc.transformer.value.MarcValueTransformers;
import org.xbib.settings.Settings;

/**
 * Indexing MARC bibliographic data in JSON Lines files into Elasticsearch.
 */
public class MarcBibliographicJSONLines extends MarcBibliographic {

    @Override
    public MarcBibliographicJSONLinesWorkerPool newPool(Settings settings) throws IOException {
        return new MarcBibliographicJSONLinesWorkerPool(settings);
    }

    public class MarcBibliographicJSONLinesWorkerPool extends MarcBibliographicWorkerPool {

        public MarcBibliographicJSONLinesWorkerPool(Settings settings) throws IOException {
            super(settings);
        }

        @Override
        public MarcBibliographicJSONLinesWorker newWorker(String name, int count) throws IOException {
            return new MarcBibliographicJSONLinesWorker(settings, this);
        }
    }

    public class MarcBibliographicJSONLinesWorker extends MarcBibliographicWorker {

        public MarcBibliographicJSONLinesWorker(Settings settings, MarcBibliographicWorkerPool pool) throws IOException {
            super(settings, pool);
        }

        @Override
        protected void doExecute(InputStream inputStream,
                                 CatalogEntityWorkerPool<CatalogEntityBuilder> catalogEntityWorkerPool,
                                 MarcValueTransformers marcValueTransformers) throws IOException {
            try (MarcXchangeJSONLinesReader reader = new MarcXchangeJSONLinesReader(inputStream,
                    settings.getAsInt("buffersize", 1024 * 1024), catalogEntityWorkerPool)) {
                reader.parse();
            }
        }
    }
}
