package org.xbib.interlibrary.catalog.elasticsearch.marc;

import java.io.IOException;
import java.io.InputStream;
import org.xbib.interlibrary.catalog.builder.MarcXchangeJSONLinesReader;
import org.xbib.interlibrary.catalog.entities.CatalogEntityBuilder;
import org.xbib.interlibrary.catalog.entities.CatalogEntityWorkerPool;
import org.xbib.marc.transformer.value.MarcValueTransformers;
import org.xbib.settings.Settings;

/**
 * Indexing MARC bibliographic data in JSON Lines files into Elasticsearch.
 */
public class MarcHoldingsJSONLines extends MarcHoldings {

    @Override
    public MarcHoldingsJSONLinesWorkerPool newPool(Settings settings) throws IOException {
        return new MarcHoldingsJSONLinesWorkerPool(settings);
    }

    public class MarcHoldingsJSONLinesWorkerPool extends MarcHoldingsWorkerPool {

        public MarcHoldingsJSONLinesWorkerPool(Settings settings) throws IOException {
            super(settings);
        }

        @Override
        public MarcHoldingsJSONLinesWorker newWorker(String name, int count) {
            return new MarcHoldingsJSONLinesWorker(settings, this);
        }
    }

    public class MarcHoldingsJSONLinesWorker extends MarcHoldingsWorker {

        public MarcHoldingsJSONLinesWorker(Settings settings, MarcHoldingsWorkerPool pool) {
            super(settings, pool);
        }

        @Override
        protected void doExecute(InputStream inputStream,
                                 CatalogEntityWorkerPool<CatalogEntityBuilder> catalogEntityWorkerPool,
                                 MarcValueTransformers marcValueTransformers) throws IOException {
            try (MarcXchangeJSONLinesReader reader = new MarcXchangeJSONLinesReader(inputStream,
                    settings.getAsInt("buffersize", 1024 * 1024), catalogEntityWorkerPool)) {
                reader.parse();
            }
        }
    }
}
